import requests
import json
from one_py_sdk.enterprise.authentication import AuthenticationApi
from one_py_sdk.shared.helpers.protobufhelper import DeserializeResponse
from one_interfaces import user_pb2 as User


class CoreApi:
    def __init__(self, env, auth: AuthenticationApi, session: requests.Session =None):
        self.AppUrl = "/enterprise/core/v1/"
        self.Environment = env
        self.Authentication = auth
        if not session:
            self.Session = requests.Session()
            self.Session.headers = {"Content-Type": "application/x-protobuf", "Accept": "application/x-protobuf"}            
        else:
            self.Session = session

    def GetUser(self, userId, expand=None):
        user = User.User()
        if (expand != None):
            url = self.Environment+self.AppUrl+"User/"+userId+"expand="+expand
        else:
            url = self.Environment+self.AppUrl+"User/"+userId
        
        response = DeserializeResponse(self.Session.get(url))
        if response.errors:
            return response
        return response.content.users.items[0]


class UserHelper:
    def GetUserFromUserInfo(userInfo):
        jResponse = json.loads(userInfo.content)



import datetime
from uuid import uuid4

class DataPoint:
    def __init__(self, stringValue: str, columnId, note: str ="", auditUserId:uuid4 ="", auditTimeStamp: datetime ="", isLocked: bool =False):
        self.stringValue = stringValue
        self.columnId = columnId
        self.note = note
        self.auditUserId = auditUserId
        self.auditTimeStamp = auditTimeStamp
        self.isLocked =isLocked
    def __repr__(self) -> str:
        return f"String value: {self.stringValue}, Column Id: {self.columnId},\
    Note: {self.note}, Audit UserId: {self.auditUserId}, Audit time: {self.auditTimeStamp}, IsLocked: {self.isLocked}"
        



import requests
import json
from logging import Logger
from one_py_sdk.shared.helpers.protobufhelper import DeserializeResponse


class ConfigurationApi:
    def __init__(self, env, auth, session: requests.Session=None):
        self.AppUrl = "/common/configuration/v2/"
        self.Environment = env
        self.Authentication = auth
        if not session:
            self.Session = requests.Session()
            self.Session.headers = {"Content-Type": "application/x-protobuf", "Accept": "application/x-protobuf"}            
        else:
            self.Session = session

    def GetSpreadsheetViews(self, authTwinId, configurationTypeId='bedc5ff2-bc8e-4916-9560-ccc28701d792'):
        url = f'{self.Environment}{self.AppUrl}?authTwinRefId={authTwinId}&configurationTypeId={configurationTypeId}'        
        response = DeserializeResponse(self.Session.get(url))
        if response.errors:
            return response
        return response.content.configurations.items



import requests
import json
from one_py_sdk.enterprise.authentication import AuthenticationApi
from one_py_sdk.shared.helpers.protobufhelper import DeserializeResponse
import uuid


class DigitalTwinApi:
    def __init__(self, env, auth: AuthenticationApi, session: requests.Session):
        self.AppUrl = "/enterprise/twin/v1/"
        self.Environment = env
        self.Authentication = auth
        if not session:
            self.Session = requests.Session()
            self.Session.headers = {
                "Content-Type": "application/x-protobuf", "Accept": "application/x-protobuf"}
        else:
            self.Session = session

    def Get(self, twinRefId: str):
        url = self.Environment+self.AppUrl+"DigitalTwin/Ref/"+twinRefId
        response = DeserializeResponse(self.Session.get(url))
        if response.errors:
            return response
        return response.content.DigitalTwins.items

    def GetTwinMeasurementsByRefId(self, twinRefId):
        url = self.Environment+self.AppUrl+"DigitalTwin/Ref/"+twinRefId
        response = DeserializeResponse(self.Session.get(url))
        if response.errors:
            return response
        return json.loads(response.content.DigitalTwins.items[0].twinData.value).get('measurement')

    def GetTwinData(self, twinRefId):
        url = self.Environment+self.AppUrl+"DigitalTwin/Ref/"+twinRefId
        response = DeserializeResponse(self.Session.get(url))
        if response.errors:
            return response
        return json.loads(response.content.DigitalTwins.items[0].twinData.value)

    def TwinMeasurements(self, twinRefId, twinMeasurements):
        try:
            usefulDictionary = {"Telemetry Id": twinRefId,
                                "Value": twinMeasurements['value'],
                                "String Value": twinMeasurements['stringValue'],
                                "Timestamp": twinMeasurements['timestamp'].get('jsonDateTime'),
                                "TelemetryPath": self.findTelemetryPath(twinRefId)}
            return usefulDictionary
        except (TypeError):
            return ("No twin data found for "+twinRefId)

    def GetDigitalTwinTypes(self):
        requestId = uuid.uuid4()
        url = self.Environment+self.AppUrl + \
            "DigitalTwinType?requestId="+str(requestId)
        response = DeserializeResponse(self.Session.get(url))
        if response.errors:
            return response
        return response.content.digitalTwinTypes.items

    def GetDigitalTwinType(self, twinTypeId: str):
        requestId = uuid.uuid4()
        url = f'{self.Environment}{self.AppUrl}DigitalTwinType/{twinTypeId}?requestId={str(requestId)}'
        response = DeserializeResponse(self.Session.get(url))
        if response.errors:
            return response
        return response.content.digitalTwinTypes.items

    def GetDigitalTwinSubtypes(self):
        requestId = uuid.uuid4()
        url = self.Environment+self.AppUrl + \
            "DigitalTwinSubType?requestId="+str(requestId)
        response = DeserializeResponse(self.Session.get(url))
        if response.errors:
            return response
        return response.content.digitalTwinSubtypes.items

    def GetDescendantsByType(self, twinRefId: str, twinTypeId: str, items=True):
        requestId = uuid.uuid4()
        url = self.Environment+self.AppUrl+"DigitalTwin/Ref/"+twinRefId + \
            "/Type/"+twinTypeId+"/Descendants?requestId="+str(requestId)

        response = DeserializeResponse(self.Session.get(url))
        if response.errors:
            return response
        if items:
            return response.content.DigitalTwins.items
        else:
            return response.content.DigitalTwins

    def GetDescendantsBySubType(self, twinRefId: str, twinSubtypeId: str):
        requestId = uuid.uuid4()
        url = self.Environment+self.AppUrl+"DigitalTwin/Ref/"+twinRefId + \
            "/Subtype/"+twinSubtypeId+"/Descendants?requestId="+str(requestId)
        response = DeserializeResponse(self.Session.get(url))
        if response.errors:
            return response
        return response.content.DigitalTwins.items

    def GetDescendants(self, twinRefId: str):
        requestId = uuid.uuid4()
        url = self.Environment+self.AppUrl+"DigitalTwin/Ref/" + \
            twinRefId+"/Descendants?requestId="+str(requestId)
        response = DeserializeResponse(self.Session.get(url))
        if response.errors:
            return response
        return response.content.DigitalTwins.items

    def GetDescendantsByRefByCategory(self, twinRefId: str, categoryId: int, items=True):
        requestId = uuid.uuid4()
        url = f'{self.Environment}{self.AppUrl}DigitalTwin/Ref/{twinRefId}/Category/{categoryId}/Descendants?requestId={str(requestId)}'
        response = DeserializeResponse(self.Session.get(url))
        if response.errors:
            return response
        if items:
            return response.content.DigitalTwins.items
        else:
            return response.content.DigitalTwins

    def FindTelemetryPath(self, twinRefId):
        statusCode = ""
        twinPath = []
        while (True):
            url = self.Environment+self.AppUrl + \
                "DigitalTwin/Ref/"+str(twinRefId)
            response = DeserializeResponse(self.Session.get(url))
            if (response.statusCode != 200):
                break
            twinRefId = response.content.DigitalTwins.items[0].parentTwinReferenceId.value
            twinPath.append(response.content.DigitalTwins.items[0].name.value)
        twinPathString = ""
        while (len(twinPath) > 0):
            twinPathString = twinPathString+str(twinPath.pop())+"/"
        return twinPathString



import requests
import json
from logging import Logger
from one_py_sdk.enterprise.authentication import AuthenticationApi
from one_py_sdk.shared.helpers.protobufhelper import DeserializeResponse


class LibraryApi:
    def __init__(self, env, auth: AuthenticationApi, session: requests.Session = None):
        self.AppUrl = "/common/library/v1/"
        self.Environment = env
        self.Authentication = auth
        if not session:
            self.Session = requests.Session()
            self.Session.headers = {"Content-Type": "application/x-protobuf", "Accept": "application/x-protobuf"}            
        else:
            self.Session = session

    def GetUnits(self):
        url = self.Environment+self.AppUrl+"unit"      
        response = DeserializeResponse(self.Session.get(url))
        if response.errors:
            return response
        return response.content.units.items

    def getParameter(self, parameterId):
        pass

    def GetParameters(self):
        url = self.Environment+self.AppUrl+"parameter"       
        r = self.Session.get(url)
        response = DeserializeResponse(r)
        if response.errors:
            return response
        return response.content.parameters.items

    def GetQuantityTypes(self):
        url = self.Environment+self.AppUrl+"quantityType"        
        response = DeserializeResponse(self.Session.get(url))
        if response.errors:
            return response
        return response.content.quantityTypes.items

    def Geti18nKeys(self,  modules: str, language: str = "en",):
        url = self.Environment+self.AppUrl+"i18n"
        if (language and modules):
            url = url+"?modulecsv="+modules+"&lang="+language
        elif (modules):
            url = url+"?modulecsv="+modules
        else:
            return print("Modules is a required parameter")
        headers = {'Authorization': self.Authentication.Token.access_token,
                   "Accept": "application/json"}
        response = requests.get(url, headers=headers)
        jResponse = json.loads(response.content)
        return jResponse.get("FM")



Environment = {"feature": "https://api-feature-us.aquaticinformatics.net", "integration": "https://api-integration-us.aquaticinformatics.net",
               "stage": "https://api-stage-us.aquaticinformatics.net", "production": "https://api-us.aquaticinformatics.net/"}



from imp import cache_from_source
from requests import Session
from datetime import time
import json
import requests_cache
from one_py_sdk.enterprise.core import CoreApi
from one_py_sdk.historian.data import HistorianApi
from one_py_sdk.operations.spreadsheet import SpreadsheetApi
from one_py_sdk.common.library import LibraryApi
from one_py_sdk.enterprise.twin import DigitalTwinApi
from one_py_sdk.enterprise.report import ReportApi
from one_py_sdk.common.configuration import ConfigurationApi
from one_py_sdk.shared.constants import *
from one_py_sdk.enterprise.authentication import AuthenticationApi
from one_py_sdk.shared.helpers.csvhelper import Exporter


class ClientSdk:
    def __init__(self, env="https://api-us.aquaticinformatics.net/", cacheTimeout: int = None):
        self.Environment = env
        self.Initialize(cacheTimeout)
    

    def Initialize(self, cacheTimeout: int):
        self.Session = Session()         
        self.Session.headers = {"Content-Type": "application/x-protobuf", "Accept": "application/x-protobuf"}            
        self.Authentication = AuthenticationApi(self.Environment, self.Session)
        self.DigitalTwin = DigitalTwinApi(
            self.Environment, self.Authentication, self.Session)
        self.Spreadsheet = SpreadsheetApi(
            self.Environment, self.Authentication, self.Session)
        self.Library = LibraryApi(self.Environment, self.Authentication, self.Session)
        self.Core = CoreApi(self.Environment, self.Authentication, self.Session)
        self.Historian = HistorianApi(self.Environment, self.Authentication, self.Session)
        self.Exporter = Exporter(self.Environment, self.Authentication, self.Session)
        self.Configuration = ConfigurationApi(
            self.Environment, self.Authentication, self.Session)
        self.Report = ReportApi(self.Environment, self.Authentication, self.Session)
        
        if cacheTimeout != None:
            requests_cache.install_cache(
                "client-cache", backend="memory", expire_after=cacheTimeout)

    def LoadCurrentUser(self):
        if not self.Authentication.IsAuthenticated:
            return print("Not authenticated. Authenticate and try again")
        if(self.Authentication.User.id != None):
            self.Authentication.GetUserInfo()
            self.Authentication.User.CopyFrom(
                self.Core.GetUser(self.Authentication.User.id))



from datetime import datetime, timezone, timedelta
import pytz as zone
from one_interfaces import jsonTicksDateTime_pb2 as jsonTicksTime
from one_interfaces import enum_timezone_pb2

timeZoneDictionary = {
    54: zone.timezone('America/Adak'),
    55: zone.timezone('America/Anchorage'),
    93: zone.timezone('America/Chicago'),
    104: zone.timezone('America/Denver'),
    146: zone.timezone('America/Los_Angeles'),
    168: zone.timezone('America/New_York'),
    179: zone.timezone('America/Phoenix'),
    184: zone.timezone('America/Puerto_Rico'),
    202: zone.timezone('America/St_Johns'),
    348: zone.timezone('Australia/Darwin'),
    349: zone.timezone("Australia/Eucla"),
    354: zone.timezone("Australia/Melbourne"),
    357: zone.timezone("Australia/Perth"),
    428: zone.timezone("Europe/Berlin"),
    441: zone.timezone("Europe/Istanbul"),
    444: zone.timezone("Europe/Kiev"),
    446: zone.timezone("Europe/Lisbon"),
    455: zone.timezone("Europe/Moscow"),
    536: zone.timezone("Pacific/Honolulu")
}

def GetRowNumber(date: datetime, wsType):
    date = date.replace(tzinfo=timezone.utc)
    BaseTime = datetime(1900, 1, 1, 0, 0, 0, 0, timezone.utc)
    diffTime = date - BaseTime
    windowSize = TimeSpanOfWorksheetType(wsType)
    diffTimeMinutes = diffTime.total_seconds()/60
    windowSizeMinutes = windowSize.total_seconds()/60
    return int(diffTimeMinutes / windowSizeMinutes) + 1


def AssumePlantTimeConvertToUtc(date: datetime, localTz: enum_timezone_pb2):
    tz = timeZoneDictionary[localTz]
    date = tz.localize(date.replace(tzinfo=None))
    utc = zone.timezone('utc')
    date = date.astimezone(utc).strftime('%Y-%m-%dT%H:%M:%S.%fZ')
    return date

def ConvertToUtc(date: datetime):   
    utc = zone.timezone('utc')
    date = date.astimezone(utc).strftime('%Y-%m-%dT%H:%M:%S.%fZ')
    return date

def GetDateFromRowNumber(rowNumber, wsType):
    BaseTime = datetime(1900, 1, 1, 0, 0, 0, 0, timezone.utc)
    row = rowNumber-1
    windowSize = TimeSpanOfWorksheetType(wsType)
    windowSizeMinutes = windowSize.total_seconds()/60
    mins = windowSizeMinutes*row
    return BaseTime+timedelta(minutes=mins)


def ToJsonTicksDateTime(date: datetime):
    jsTime = jsonTicksTime.JsonTicksDateTime()
    jsTime.jsonDateTime.value = str(date)
    return jsTime


def TimeSpanOfWorksheetType(wsType):
    if wsType == 1:
        return timedelta(minutes=15)
    elif wsType == 2:
        return timedelta(hours=1)
    elif wsType == 3:
        return timedelta(hours=4)
    elif wsType == 4:
        return timedelta(days=1)
    else:
        return "Invalid worksheet type"



from one_interfaces import apierror_pb2 as apiError
from one_interfaces import apiresponse_pb2 as apiResponse
import google
import logging


def DeserializeResponse(response):
    try:
        pbResponse = apiResponse.ApiResponse()
        if response.status_code == 404:
            pbResponse.statusCode = 404
            error = apiError.ApiError()
            error.statusCode =404
            error.detail="Not found"            
            pbResponse.errors.append(error)            
            return pbResponse
        if response.status_code == 401:
            pbResponse.statusCode = 401            
            error = apiError.ApiError()
            error.statusCode =401
            error.detail="Not authenticated"            
            pbResponse.errors.append(error)            
            return pbResponse
        if response.status_code == 403:
            pbResponse.statusCode = 403
            error = apiError.ApiError()
            error.statusCode =403
            error.detail="Not authorized for this resource"  
            pbResponse.errors.add(error)
            return pbResponse
        pbResponse.ParseFromString(response.content)
        return pbResponse
    except Exception as Argument:
        logging.exception("Error occured in deserialization process", Argument)
        



from requests import Session
import json
import logging
from one_py_sdk.enterprise.authentication import AuthenticationApi
from one_py_sdk.shared.helpers.protobufhelper import DeserializeResponse
from one_py_sdk.shared.helpers.helpers import IsValidGuid


class ReportApi:
    def __init__(self, env, auth: AuthenticationApi, session: Session= None ):
        self.AppUrl = "/enterprise/report/v1/"
        self.Environment = env
        self.Authentication = auth
        if not session:
            self.Session = Session()
            self.Session.headers = {"Content-Type": "application/x-protobuf", "Accept": "application/x-protobuf"}            
        else:
            self.Session = session

    def GetReportDefinitions(self, plantId=None):
        url = f'{self.Environment}{self.AppUrl}definitions'
        if plantId and IsValidGuid(plantId):
            url = url+f"?plantId={plantId}"        
        response = DeserializeResponse(self.Session.get(url))
        if response.errors:
            return response
        return response.content.ReportDefinitions.items

    def GetColumnIdsByPlant(self, plantId):
        if not plantId:
            return []
        if not IsValidGuid(plantId):
            return logging.error(f"{plantId} is not a valid guid")
        reports = self.GetReportDefinitions(plantId)
        try:
            reportDefs = [json.loads(report.reportDefinitionJson.value)
                          for report in reports]
        except:
            return reports
        columns = [d.get('columns') for d in reportDefs]
        ids = []
        for colLst in columns:
            for col in colLst:
                ids.append(col.get('id'))
        uniqueIds = []
        for id in ids:
            if id not in uniqueIds:
                uniqueIds.append(id)
        return uniqueIds

    def GetReportDefinitionById(self, reportId):
        if not reportId:
            return []
        if not IsValidGuid(reportId):
            return logging.error(f"{reportId} is not a valid guid")
        url = f'{self.Environment}{self.AppUrl}definitions/{reportId}'        
        response = DeserializeResponse(self.Session.get(url))
        if response.errors:
            return response
        elif response.statusCode == 404:
            return response
        return response.content.ReportDefinitions.items

    def GetColumnIdsByReportId(self, reportId):
        if not IsValidGuid(reportId):
            return logging.error(f"{reportId} is not a valid guid")
        reports = self.GetReportDefinitionById(reportId)
        try:
            reportDefs = [json.loads(report.reportDefinitionJson.value)
                          for report in reports]
        except:
            return reports
        columns = [d.get('columns') for d in reportDefs]
        ids = []
        for colLst in columns:
            for col in colLst:
                ids.append(col.get('id'))
        return ids

    def GetColumnIdsByReportName(self, reportName, plantId=None):
        if not reportName:
            return []
        if not plantId or not IsValidGuid(plantId):
            logging.warning(f"PlantId :{plantId} was either not supplied or invalid. Column Ids associated with all reports this user has access to with this name will be returned.")        
        reports = self.GetReportDefinitions(plantId)
        try:
            reportDefs = [json.loads(report.reportDefinitionJson.value)
                          for report in reports if report.name.value == reportName]
        except:
            return reports
        columns = [d.get('columns') for d in reportDefs]
        ids = []
        for colLst in columns:
            for col in colLst:
                ids.append(col.get('id'))
        return ids



from collections import OrderedDict
from datetime import datetime
import uuid
from requests import Session
from one_py_sdk.enterprise.authentication import AuthenticationApi
from one_py_sdk.shared.helpers.protobufhelper import DeserializeResponse
from one_py_sdk.shared.helpers.datetimehelper import *
from one_py_sdk.shared.helpers.helpers import IsValidGuid
from one_py_sdk.shared.models.datapoint import DataPoint
from one_interfaces import row_pb2 as row
from one_interfaces import cell_pb2 as cell
from one_interfaces import celldata_pb2 as celldata
from one_interfaces import auditevent_pb2 as audit
from one_interfaces import note_pb2 as note


class SpreadsheetApi:
    def __init__(self, env: str, auth: AuthenticationApi, session: Session = None):
        self.Environment = env
        self.Auth = auth
        self.AppUrl = "/operations/spreadsheet/v1/"
        if not session:
            self.Session = Session()
            self.Session.headers = {
                "Content-Type": "application/x-protobuf", "Accept": "application/x-protobuf"}
        else:
            self.Session = session

    def ImportDictionary(self, plantId, valueDict, wsType):
        rows = self.__rowBuilder(valueDict, wsType, plantId)
        return self.SaveRows(plantId, wsType, rows)

    def SaveRows(self, plantId, wsType, rows):
        url = f"{self.Environment}{self.AppUrl}{plantId}/worksheet/{str(wsType)}/rows"
        response = DeserializeResponse(self.Session.post(
            url, data=rows.SerializeToString()))
        return response

    def __rowBuilder(self, valueDict: dict[datetime: [DataPoint]], wsType, plantId):
        r = row.Rows()
        spreadsheetDef = self.GetSpreadsheetDefinition(plantId)
        sortedValueDict = OrderedDict(valueDict.items())
        map = self.MapColumnGuidToIntId(plantId, wsType)
        rowNumbers = []
        for key in valueDict.keys():
            rowNumber = GetRowNumber(key, wsType)
            utcTime = AssumePlantTimeConvertToUtc(key, spreadsheetDef[0].enumTimeZone)            
            for dataPoint in sortedValueDict[key]:
                cd = celldata.CellData()
                c = cell.Cell()
                if IsValidGuid(dataPoint.columnId):
                    c.columnNumber = map[dataPoint.columnId]
                else:
                    c.columnNumber = dataPoint.columnId
                try:
                    if dataPoint.note != "":
                        n = note.Note()
                        n.text = dataPoint.note
                        if dataPoint.auditUserId != "":
                            n.userId = dataPoint.auditUserId
                        else:
                            n.userId = self.Auth.User.id
                        if dataPoint.auditTimeStamp != "":
                            n.timeStamp.jsonDateTime.value = ToJsonTicksDateTime(
                                AssumePlantTimeConvertToUtc(dataPoint.auditTimeStamp, spreadsheetDef[0].enumTimeZone)).jsonDateTime.value
                        else:
                            n.timeStamp.jsonDateTime.value = ToJsonTicksDateTime(
                                utcTime).jsonDateTime.value
                        c.notes.append(n)
                except:
                    pass
                s = audit.AuditEvent()
                s.id = str(uuid.uuid4())
                if dataPoint.auditUserId != "":
                    s.userId = dataPoint.auditUserId
                    cd.dataSourceBinding.bindingId = dataPoint.auditUserId
                else:
                    s.userId = self.Auth.User.id
                    cd.dataSourceBinding.bindingId = self.Auth.User.id
                if dataPoint.auditTimeStamp != "":
                    s.timeStamp.jsonDateTime.value = ToJsonTicksDateTime(
                        AssumePlantTimeConvertToUtc(dataPoint.auditTimeStamp, spreadsheetDef[0].enumTimeZone)).jsonDateTime.value
                else:
                    s.timeStamp.jsonDateTime.value = ToJsonTicksDateTime(
                        utcTime).jsonDateTime.value
                s.details = "Python SDK import"
                s.enumDataSource = 5
                s.enumDomainSource = 2
                cd.isLocked = dataPoint.isLocked
                cd.auditEvents.append(s)
                cd.stringValue.value = dataPoint.stringValue
                cd.dataSourceBinding.dataSource = 5
                cd.dataSourceBinding.enumSamplingStatistic = 0
                if rowNumber not in rowNumbers:
                    rowNumbers.append(rowNumber)
                    r2 = row.Row()
                    r2.rowNumber = rowNumber
                    c.cellDatas.append(cd)
                    r2.cells.append(c)
                    r.items[r2.rowNumber].CopyFrom(r2)
                else:
                    itemAdded = False
                    r2 = r.items[rowNumber]
                    for item in r2.cells:
                        if item.columnNumber == c.columnNumber:
                            item.cellDatas.insert(0, cd)
                            itemAdded = True
                    if not itemAdded:
                        c.cellDatas.append(cd)
                        r2.cells.append(c)
        return r

    def GetWorksheetColumnIds(self, plantId, wsType):
        url = self.Environment + self.AppUrl + plantId + \
            "/worksheet/"+str(wsType)+"/definition"
        response = DeserializeResponse(self.Session.get(url))
        if response.errors:
            return response
        columnIds = [
            col.columnId for col in response.content.worksheetDefinitions.items[0].columns if col.isActive == True]
        return columnIds

    def MapColumnGuidToIntId(self, plantId, wsType):
        url = self.Environment + self.AppUrl + plantId + \
            "/worksheet/"+str(wsType)+"/definition"
        response = DeserializeResponse(self.Session.get(url))
        if response.errors:
            return response
        guidToIntMap = {}
        for col in response.content.worksheetDefinitions.items[0].columns:
            guidToIntMap[col.columnId] = col.columnNumber
        return guidToIntMap

    def GetWorksheetColumnNumbers(self, plantId, wsType):
        url = self.Environment + self.AppUrl + plantId + \
            "/worksheet/"+str(wsType)+"/definition"
        response = DeserializeResponse(self.Session.get(url))
        if response.errors:
            return response
        columnNumbers = [
            col.columnNumber for col in response.content.worksheetDefinitions.items[0].columns if col.isActive == True]
        return columnNumbers

    def GetWorksheetDefinition(self, plantId, wsType):
        url = self.Environment + self.AppUrl + plantId + \
            "/worksheet/"+str(wsType)+"/definition"
        response = DeserializeResponse(self.Session.get(url))
        if response.errors:
            return response
        return response.content.worksheetDefinitions.items

    def GetSpreadsheetDefinition(self, plantId):
        url = f'{self.Environment}{self.AppUrl}{plantId}/definition'
        response = DeserializeResponse(self.Session.get(url))
        if response.errors:
            return response
        return response.content.spreadsheetDefinitions.items

    def GetColumnByDay(self, plantId, wsType, columnId, date: datetime):
        url = self.Environment + self.AppUrl + plantId + \
            f'/worksheet/{str(wsType)}/column/{columnId}/byday/{date.year}/{date.month}/{date.day}'
        response = DeserializeResponse(self.Session.get(url))
        if response.errors:
            return response
        return response.content.measurements.items

    def GetColumnByMonth(self, plantId: str, wsType: int, columnId: int, date: datetime):
        url = self.Environment + self.AppUrl + plantId + \
            f'/worksheet/{str(wsType)}/column/{columnId}/bymonth/{date.year}/{date.month}'
        response = DeserializeResponse(self.Session.get(url))
        if response.errors:
            return response
        return response.content.measurements.items

    def GetColumnByYear(self, plantId, wsType, columnId, date: datetime):
        url = self.Environment + self.AppUrl + plantId + \
            f'/worksheet/{str(wsType)}/column/{columnId}/byyear/{date.year}'
        response = DeserializeResponse(self.Session.get(url))
        if response.errors:
            return response
        return response.content.measurements.items

    def GetRows(self, plantId, wsType, startRow=None, endRow=None, columns=None, viewId=None):
        if columns and viewId:
            return print("Using both columns and viewId parameters together is not supported.")
        requestId = uuid.uuid4()
        url = f'{self.Environment}{self.AppUrl}{plantId}/worksheet/{str(wsType)}/rows?requestId={requestId}'
        if startRow:
            url = url+f'&startRow={startRow}'
        if endRow:
            url = url+f'&endRow={endRow}'
        if columns:
            url = url+f'&columns={columns}'
        if viewId:
            url = url+f'&viewId={viewId}'

        response = DeserializeResponse(self.Session.get(url))
        if response.errors:
            return response
        return response.content.rows.items

    def __getRows(self, plantId, wsType, startRow=None, endRow=None, columns=None, viewId=None):
        if columns and viewId:
            return print("Using both columns and viewId parameters together is not supported.")
        requestId = uuid.uuid4()
        url = f'{self.Environment}{self.AppUrl}{plantId}/worksheet/{str(wsType)}/rows?requestId={requestId}'
        if startRow:
            url = url+f'&startRow={startRow}'
        if endRow:
            url = url+f'&endRow={endRow}'
        if columns:
            url = url+f'&columns={columns}'
        if viewId:
            url = url+f'&viewId={viewId}'

        response = DeserializeResponse(self.Session.get(url))
        if response.errors:
            return response
        return response.content.rows

    def GetRowsByDay(self, plantId, wsType, date: datetime, columns=None, viewId=None):
        if columns and viewId:
            return print("Using both columns and viewId parameters together is not supported.")
        url = self.Environment + self.AppUrl + \
            f'{plantId}/worksheet/{str(wsType)}/rows/byday/{date.year}/{date.month}/{date.day}'
        response = DeserializeResponse(self.Session.get(url))
        if response.errors:
            return response
        return response.content.rows.items

    def GetRowsByMonth(self, plantId, wsType, date: datetime, columns=None, viewId=None):
        if columns and viewId:
            return print("Using both columns and viewId parameters together is not supported.")
        url = self.Environment + self.AppUrl + \
            f'{plantId}/worksheet/{str(wsType)}/rows/bymonth/{date.year}/{date.month}'
        response = DeserializeResponse(self.Session.get(url))
        if response.errors:
            return response
        return response.content.rows.items

    def GetRowsForTimeRange(self, plantId, wsType, startDate: datetime, endDate: datetime):
        startRow = GetRowNumber(startDate, wsType)
        endRow = GetRowNumber(endDate, wsType)
        rows = row.Rows()
        while endRow - startRow > 5000:
            newEndRow = startRow+5000
            rows.MergeFrom(self.__getRows(
                plantId, wsType, startRow, newEndRow))
            startRow = newEndRow+1
        rows.MergeFrom(self.__getRows(plantId, wsType, startRow, endRow))
        return rows.items



from datetime import datetime
import requests
import json
from one_py_sdk.enterprise.authentication import AuthenticationApi
from one_py_sdk.shared.helpers.protobufhelper import DeserializeResponse


class HistorianApi:
    def __init__(self, env, auth: AuthenticationApi, session: requests.Session=None):
        self.Environment = env
        self.Authentication = auth
        self.AppUrl = "/historian/data/v1/"
        if not session:
            self.Session = requests.Session()
            self.Session.headers = {"Content-Type": "application/x-protobuf", "Accept": "application/x-protobuf"}            
        else:
            self.Session = session

    def GetHistorianData(self, twinRefId, date: datetime):
        date = f'{str(date.date())}T{str(date.time())}Z'
        url = f'{self.Environment}{self.AppUrl}{twinRefId}/{date}'
        response = DeserializeResponse(self.Session.get(url))
        if response.errors:
            return response
        return response.content.historianDatas.items

    def GetHistorianDataRange(self, twinRefId, startTime: datetime, endTime: datetime):
        startString = f'{str(startTime.date())}T{str(startTime.time())}'
        endString = f'{str(endTime.date())}T{str(endTime.time())}'
        url = f'{self.Environment}{self.AppUrl}{twinRefId}?startTime={startString}&endTime={endString}'
        response = DeserializeResponse(self.Session.get(url))
        if response.errors:
            return response
        return response.content.historianDatas.items
